package java.text;

import java.math.BigInteger;
import sun.misc.FloatingDecimal;
import java.math.BigDecimal;
import java.math.RoundingMode;

final class DigitList implements Cloneable
{
    public static final int MAX_COUNT = 19;
    public int decimalAt;
    public int count;
    public char[] digits;
    private char[] data;
    private RoundingMode roundingMode;
    private boolean isNegative;
    private static final char[] LONG_MIN_REP;
    private StringBuffer tempBuffer;
    
    DigitList() {
        this.decimalAt = 0;
        this.count = 0;
        this.digits = new char[19];
        this.roundingMode = RoundingMode.HALF_EVEN;
        this.isNegative = false;
    }
    
    boolean isZero() {
        int i = 0;
        while (i < this.count) {
            if (this.digits[i] != '0') {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    void setRoundingMode(RoundingMode roundingMode) {
        this.roundingMode = roundingMode;
    }
    
    public void clear() {
        this.decimalAt = 0;
        this.count = 0;
    }
    
    public void append(char c) {
        if (this.count == this.digits.length) {
            char[] digits = new char[this.count + 100];
            System.arraycopy(this.digits, 0, digits, 0, this.count);
            this.digits = digits;
        }
        this.digits[this.count++] = c;
    }
    
    public final double getDouble() {
        if (this.count == 0) {
            return 0.0;
        }
        StringBuffer stringBuffer = this.getStringBuffer();
        stringBuffer.append('.');
        stringBuffer.append(this.digits, 0, this.count);
        stringBuffer.append('E');
        stringBuffer.append(this.decimalAt);
        return Double.parseDouble(stringBuffer.toString());
    }
    
    public final long getLong() {
        if (this.count == 0) {
            return 0L;
        }
        if (this.isLongMIN_VALUE()) {
            return Long.MIN_VALUE;
        }
        StringBuffer stringBuffer = this.getStringBuffer();
        stringBuffer.append(this.digits, 0, this.count);
        for (int i = this.count; i < this.decimalAt; ++i) {
            stringBuffer.append('0');
        }
        return Long.parseLong(stringBuffer.toString());
    }
    
    public final BigDecimal getBigDecimal() {
        if (this.count == 0) {
            if (this.decimalAt == 0) {
                return BigDecimal.ZERO;
            }
            return new BigDecimal("0E" + this.decimalAt);
        }
        else {
            if (this.decimalAt == this.count) {
                return new BigDecimal(this.digits, 0, this.count);
            }
            return new BigDecimal(this.digits, 0, this.count).scaleByPowerOfTen(this.decimalAt - this.count);
        }
    }
    
    boolean fitsIntoLong(boolean b, boolean b2) {
        while (this.count > 0 && this.digits[this.count - 1] == '0') {
            --this.count;
        }
        if (this.count == 0) {
            return b || b2;
        }
        if (this.decimalAt < this.count || this.decimalAt > 19) {
            return false;
        }
        if (this.decimalAt < 19) {
            return true;
        }
        int i = 0;
        while (i < this.count) {
            int n = this.digits[i];
            int n2 = DigitList.LONG_MIN_REP[i];
            if (n > n2) {
                return false;
            }
            if (n < n2) {
                return true;
            }
            ++i;
        }
        return this.count < this.decimalAt || !b;
    }
    
    final void set(boolean b, double n, int n2) {
        this.set(b, n, n2, true);
    }
    
    final void set(boolean b, double n, int n2, boolean b2) {
        FloatingDecimal.BinaryToASCIIConverter binaryToASCIIConverter = FloatingDecimal.getBinaryToASCIIConverter(n);
        int digitsRoundedUp = binaryToASCIIConverter.digitsRoundedUp();
        int decimalDigitsExact = binaryToASCIIConverter.decimalDigitsExact();
        assert !binaryToASCIIConverter.isExceptional();
        this.set(b, binaryToASCIIConverter.toJavaFormatString(), (boolean)digitsRoundedUp, (boolean)decimalDigitsExact, n2, b2);
        return;
    }
    
    private void set(boolean isNegative, String s, boolean b, boolean b2, int n, boolean b3) {
        this.isNegative = isNegative;
        int length = s.length();
        char[] dataChars = this.getDataChars(length);
        s.getChars(0, length, dataChars, 0);
        this.decimalAt = -1;
        this.count = 0;
        int int1 = 0;
        int n2 = 0;
        boolean n3 = false;
        int i = 0;
        while (i < length) {
            int n4 = dataChars[i++];
            if (n4 == '.') {
                this.decimalAt = this.count;
            }
            else {
                if (n4 == 'e' || n4 == 'E') {
                    int1 = parseInt(dataChars, i, length);
                    break;
                }
                if (n3 == 0) {
                    n3 = (n4 != '0');
                    if (n3 == 0 && this.decimalAt != -1) {
                        ++n2;
                    }
                }
                if (n3 != 0) {
                    this.digits[this.count++] = n4;
                }
                else {
                    continue;
                }
            }
        }
        if (this.decimalAt == -1) {
            this.decimalAt = this.count;
        }
        if (n3 != 0) {
            this.decimalAt += int1 - n2;
        }
        if (b3) {
            if (-this.decimalAt > n) {
                this.count = 0;
                return;
            }
            if (-this.decimalAt == n) {
                if (this.shouldRoundUp(0, b, b2)) {
                    this.count = 1;
                    ++this.decimalAt;
                    this.digits[0] = '1';
                }
                else {
                    this.count = 0;
                }
                return;
            }
        }
        while (this.count > 1 && this.digits[this.count - 1] == '0') {
            --this.count;
        }
        this.round(b3 ? (n + this.decimalAt) : n, b, b2);
    }
    
    private final void round(int count, boolean b, boolean b2) {
        if (count >= 0 && count < this.count) {
            Label_0080: {
                if (this.shouldRoundUp(count, b, b2)) {
                    Label_0077: {
                        while (--count >= 0) {
                            char[] digits = this.digits;
                            int n = count;
                            ++digits[n];
                            if (this.digits[count] > '9') {
                                continue;
                            }
                            ++count;
                            break Label_0080;
                        }
                        this.digits[0] = '1';
                        ++this.decimalAt;
                        count = 0;
                        continue;
                    }
                }
            }
            this.count = count;
            while (this.count > 1 && this.digits[this.count - 1] == '0') {
                --this.count;
            }
        }
    }
    
    private boolean shouldRoundUp(int i, boolean b, boolean b2) {
        if (i < this.count) {
            switch (this.roundingMode) {
                case UP: {
                    while (i < this.count) {
                        if (this.digits[i] != '0') {
                            return true;
                        }
                        ++i;
                    }
                    break;
                }
                case DOWN: {
                    break;
                }
                case CEILING: {
                    while (i < this.count) {
                        if (this.digits[i] != '0') {
                            return !this.isNegative;
                        }
                        ++i;
                    }
                    break;
                }
                case FLOOR: {
                    while (i < this.count) {
                        if (this.digits[i] != '0') {
                            return this.isNegative;
                        }
                        ++i;
                    }
                    break;
                }
                case HALF_UP:
                case HALF_DOWN: {
                    if (this.digits[i] > '5') {
                        return true;
                    }
                    if (this.digits[i] == '5') {
                        if (i != this.count - 1) {
                            return true;
                        }
                        if (b2) {
                            return this.roundingMode == RoundingMode.HALF_UP;
                        }
                        return !b;
                    }
                    else {
                        break;
                    }
                }
                case HALF_EVEN: {
                    if (this.digits[i] > '5') {
                        return true;
                    }
                    if (this.digits[i] == '5') {
                        if (i == this.count - 1) {
                            return !b && (!b2 || (i > 0 && this.digits[i - 1] % 2 != 0));
                        }
                        int j = i + 1;
                        while (j < this.count) {
                            if (this.digits[j] != '0') {
                                return true;
                            }
                            ++j;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case UNNECESSARY: {
                    while (i < this.count) {
                        if (this.digits[i] != '0') {
                            throw new ArithmeticException("Rounding needed with the rounding mode being set to RoundingMode.UNNECESSARY");
                        }
                        ++i;
                    }
                    break;
                }
                default: {
                    assert false;
                    break;
                }
            }
        }
        return false;
    }
    
    final void set(boolean b, long n) {
        this.set(b, n, 0);
    }
    
    final void set(boolean isNegative, long n, int n2) {
        this.isNegative = isNegative;
        if (n <= 0L) {
            if (n == Long.MIN_VALUE) {
                int n6 = 19;
                this.count = n6;
                this.decimalAt = n6;
                System.arraycopy(DigitList.LONG_MIN_REP, 0, this.digits, 0, this.count);
            }
            else {
                int n5 = 0;
                this.count = n5;
                this.decimalAt = n5;
            }
        }
        else {
            int n3 = 19;
            while (n > 0L) {
                this.digits[--n3] = (char)(int)(48L + n % 10L);
                n /= 10L;
            }
            this.decimalAt = 19 - n3;
            int n4;
            for (n4 = 18; this.digits[n4] == '0'; --n4) {}
            this.count = n4 - n3 + 1;
            System.arraycopy(this.digits, n3, this.digits, 0, this.count);
        }
        if (n2 > 0) {
            this.round(n2, false, true);
        }
    }
    
    final void set(boolean b, BigDecimal bigDecimal, int n, boolean b2) {
        String string = bigDecimal.toString();
        this.extendDigits(string.length());
        this.set(b, string, false, true, n, b2);
    }
    
    final void set(boolean isNegative, BigInteger bigInteger, int n) {
        this.isNegative = isNegative;
        String string = bigInteger.toString();
        int length = string.length();
        this.extendDigits(length);
        string.getChars(0, length, this.digits, 0);
        this.decimalAt = length;
        int n2;
        for (n2 = length - 1; n2 >= 0 && this.digits[n2] == '0'; --n2) {}
        this.count = n2 + 1;
        if (n > 0) {
            this.round(n, false, true);
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof DigitList)) {
            return false;
        }
        DigitList list = (DigitList)o;
        if (this.count != list.count || this.decimalAt != list.decimalAt) {
            return false;
        }
        int i = 0;
        while (i < this.count) {
            if (this.digits[i] != list.digits[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int decimalAt = this.decimalAt;
        for (int i = 0; i < this.count; ++i) {
            decimalAt = decimalAt * 37 + this.digits[i];
        }
        return decimalAt;
    }
    
    public Object clone() {
        try {
            DigitList list = (DigitList)super.clone();
            char[] digits = new char[this.digits.length];
            System.arraycopy(this.digits, 0, digits, 0, this.digits.length);
            list.digits = digits;
            list.tempBuffer = null;
            return list;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    private boolean isLongMIN_VALUE() {
        if (this.decimalAt != this.count || this.count != 19) {
            return false;
        }
        int i = 0;
        while (i < this.count) {
            if (this.digits[i] != DigitList.LONG_MIN_REP[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static final int parseInt(char[] array, int i, int n) {
        boolean n2 = true;
        int n3;
        if ((n3 = array[i]) == '-') {
            n2 = false;
            ++i;
        }
        else if (n3 == '+') {
            ++i;
        }
        int n4 = 0;
        while (i < n) {
            int n5 = array[i++];
            if (n5 >= '0' && n5 <= '9') {
                n4 = n4 * 10 + (n5 - 48);
            }
            else {
                break;
            }
        }
        return n2 ? n4 : (-n4);
    }
    
    @Override
    public String toString() {
        if (this.isZero()) {
            return "0";
        }
        StringBuffer stringBuffer = this.getStringBuffer();
        stringBuffer.append("0.");
        stringBuffer.append(this.digits, 0, this.count);
        stringBuffer.append("x10^");
        stringBuffer.append(this.decimalAt);
        return stringBuffer.toString();
    }
    
    private StringBuffer getStringBuffer() {
        if (this.tempBuffer == null) {
            this.tempBuffer = new StringBuffer(19);
        }
        else {
            this.tempBuffer.setLength(0);
        }
        return this.tempBuffer;
    }
    
    private void extendDigits(int n) {
        if (n > this.digits.length) {
            this.digits = new char[n];
        }
    }
    
    private final char[] getDataChars(int n) {
        if (this.data == null || this.data.length < n) {
            this.data = new char[n];
        }
        return this.data;
    }
    
    static {
        LONG_MIN_REP = "9223372036854775808".toCharArray();
    }
}
